<?php

declare(strict_types=1);

namespace Drupal\dwarl_mailer\EventSubscriber;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\dwarl\ContactMethod;
use Drupal\dwarl\Event\SendOtpEvent;
use Drupal\dwarl_mailer\EmailManagerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Dwarl mailer send OTP event subscriber.
 */
class SendOtpEventSubscriber implements EventSubscriberInterface {
  use StringTranslationTrait;

  /**
   * SendOtpEventSubscriber constructor.
   */
  public function __construct(
    private readonly EmailManagerInterface $emailManager,
  ) {
  }

  /**
   * {@inheritDoc}
   */
  public static function getSubscribedEvents(): array {
    $events = [];
    $events[SendOtpEvent::SEND_OTP][] = ['sendOtp'];
    return $events;
  }

  /**
   * Send otp event.
   *
   * Store the plain otp in the db so the test script can get it.
   *
   * @param \Drupal\dwarl\Event\SendOtpEvent $sendOtpEvent
   *   The send otp event.
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   */
  public function sendOtp(SendOtpEvent $sendOtpEvent): void {
    $otp = $sendOtpEvent->getOtp();
    $email = $sendOtpEvent->getEmail();
    $phone = $sendOtpEvent->getPhone();
    $method = $sendOtpEvent->getMethod();

    $args = ['@reason' => $sendOtpEvent->getReason()->value];
    $text = (string) $this->t('Your one time code to @reason is:', $args);

    if ($method === ContactMethod::Email) {
      $this->emailManager->sendCodeEmail($email, $otp, $text);
    }
  }

}
