<?php

declare(strict_types=1);

namespace Drupal\dwarl_phone\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\dwarl\Register;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Phone controller.
 */
final class PhoneController extends ControllerBase {

  /**
   * PhoneController constructor.
   */
  public function __construct(
    private readonly Request $request,
    private readonly Register $register,
  ) {}

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container): self {

    /** @var \Symfony\Component\HttpFoundation\RequestStack $requestStack */
    $requestStack = $container->get('request_stack');

    /** @var \Drupal\dwarl\Register $register */
    $register = $container->get('dwarl.register');

    return new self(
      $requestStack->getCurrentRequest(),
      $register,
    );
  }

  /**
   * Register new user.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The response.
   */
  public function registerPhone(): JsonResponse {
    $phone = $this->request->get('phone');
    $country = $this->request->get('country');

    if (empty($phone)) {
      return new JsonResponse(['error' => 'Phone is required'], 422);
    }
    if (empty($country)) {
      return new JsonResponse(['error' => 'Country is required'], 422);
    }

    try {
      $sessionId = $this->register->registerPhone($phone, $country);
    }
    catch (\Exception $exception) {
      $response = ['error' => $exception->getMessage()];
      return new JsonResponse($response, 400);
    }

    $response = new JsonResponse();
    $response->headers->set('session-id', $sessionId);
    $response->headers->set('Access-Control-Expose-Headers', ['session-id']);

    return $response;
  }

}
