<?php

declare(strict_types=1);

namespace Drupal\dwarl_sms_twilio\EventSubscriber;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\dwarl\ContactMethod;
use Drupal\dwarl\Event\SendOtpEvent;
use Drupal\dwarl_mailer\EmailManagerInterface;
use Drupal\dwarl_sms_twilio\SenderInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Dwarl mailer send OTP event subscriber.
 */
class SendOtpEventSubscriber implements EventSubscriberInterface {
  use StringTranslationTrait;

  /**
   * SendOtpEventSubscriber constructor.
   *
   * @param \Drupal\dwarl_sms_twilio\SenderInterface $sender
   *   The sender service.
   */
  public function __construct(
    private readonly SenderInterface $sender,
  ) {
  }

  /**
   * {@inheritDoc}
   */
  public static function getSubscribedEvents(): array {
    $events = [];
    $events[SendOtpEvent::SEND_OTP][] = ['sendOtp'];
    return $events;
  }

  /**
   * Send otp event.
   *
   * Store the plain otp in the db so the test script can get it.
   *
   * @param \Drupal\dwarl\Event\SendOtpEvent $sendOtpEvent
   *   The send otp event.
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   */
  public function sendOtp(SendOtpEvent $sendOtpEvent): void {
    $otp = $sendOtpEvent->getOtp();
    $phone = $sendOtpEvent->getPhone();
    $method = $sendOtpEvent->getMethod();

    $logger = \Drupal::logger('dwarl_sms_twilio');
    $logger->notice('sendOtp');

    if ($method === ContactMethod::Phone) {
      $args = [
        '@reason' => $sendOtpEvent->getReason()->value,
        '@otp' => $otp,
      ];
      $text = (string) $this->t('Your one time code to @reason is: @otp', $args);
      $this->sender->sendMessage($phone, $text);
    }
  }

}
