<?php

declare(strict_types=1);

namespace Drupal\dwarl_sms_twilio;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Utility\Error;
use Drupal\key\KeyRepositoryInterface;
use Twilio\Rest\Client;

/**
 * Sender service to send SMS messages.
 */
final class Sender implements SenderInterface {

  public function __construct(
    private readonly KeyRepositoryInterface $keyRepository,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public function sendMessage(string $number, string $message): void {

    $logger = \Drupal::logger('dwarl_sms_twilio');

    $key = $this->keyRepository->getKey('twilio_account');

    if (!$key) {
      throw new \Exception('twilio_account key not found.');
    }

    $value = Json::decode($key->getKeyValue());

    if (!isset($value['sid'], $value['token'])) {
      throw new \Exception('twilio_account key must contain sid and token.');
    }

    $sid = $value['sid'];
    $token = $value['token'];

    try {
      $client = new Client($sid, $token);

      $client->messages->create(
        $number,
        [
          'from' => '+44 7888 872702',
          'body' => $message,
        ]
      );
      $logger->notice('Sent sms to: ' . $number);
    }
    catch (\Exception $exception) {
      Error::logException($logger, $exception);
    }

  }

}
