<?php

declare(strict_types=1);

namespace Drupal\dwarl\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Event\ExceptionEvent;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Dwarl final exception handler.
 */
class DwarlExceptionSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[KernelEvents::EXCEPTION][] = ['onException', -40];
    return $events;
  }

  /**
   * Return json response for errors.
   *
   * @param \Symfony\Component\HttpKernel\Event\ExceptionEvent $event
   *   The event to process.
   */
  public function onException(ExceptionEvent $event): void {
    $request = $event->getRequest();
    if (str_starts_with($request->getPathInfo(), '/dwarl/') === FALSE) {
      return;
    }

    $exception = $event->getThrowable();

    if ($exception instanceof HttpException) {
      $body = ['error' => $exception->getMessage()];
      $response = new JsonResponse($body, $exception->getStatusCode());
      $response->setStatusCode($exception->getStatusCode());
      $event->setResponse($response);
    }
    else {
      // @todo switch this back.
      // $response = ['error' => 'Internal Server Error'];
      $response = [
        'error' => $exception->getMessage(),
        'file' => $exception->getFile(),
        'line' => $exception->getLine(),
        'trace' => $exception->getTraceAsString(),
      ];
      $response = new JsonResponse($response, Response::HTTP_INTERNAL_SERVER_ERROR);
      $response->setStatusCode(500);
      $event->setResponse($response);
    }
  }

}
