<?php

declare(strict_types=1);

namespace Drupal\dwarl\Otp;

use Drupal\Core\TempStore\PrivateTempStore;
use Drupal\dwarl\ContactMethod;
use Drupal\dwarl\Store\StoreInterface;
use Drupal\user\UserInterface;

/**
 * @todo Add interface description.
 */
interface OtpServiceInterface {

  /**
   * Generate one time pass.
   *
   * 1. Generate code.
   * 2. Save in store.
   * 3. Trigger send otp event.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStore|StoreInterface $store
   *   The store.
   * @param \Drupal\user\UserInterface $user
   *   The user.
   * @param \Drupal\dwarl\Otp\OtpSendReason $reason
   *   The event name, just used for logging.
   * @param \Drupal\dwarl\ContactMethod|null $otpSendMethod
   *   The otp send method or null to use default.
   * @param string|null $emailPhoneOverride
   *   Use this instead of users saved email / phone.
   *
   * @throws \Drupal\dwarl\Exception\FailedToGenerateOtpException
   */
  public function generateOtp(
    PrivateTempStore|StoreInterface $store,
    UserInterface $user,
    OtpSendReason $reason,
    ?ContactMethod $otpSendMethod = NULL,
    ?string $emailPhoneOverride = NULL,
  ): void;

  /**
   * Validate one time pass from request.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStore|\Drupal\dwarl\Store\StoreInterface $store
   *   The store.
   * @param string|null $otp
   *   The otp to validate. If null gets it from the request.
   *
   * @throws \Drupal\dwarl\Exception\InvalidOtpException
   * @throws \Drupal\dwarl\Exception\OtpBlockedException
   */
  public function validateOtp(PrivateTempStore|StoreInterface $store, ?string $otp = NULL): void;

  /**
   * Validate contact method.
   *
   * @param string|null $method
   *   The method.
   *
   * @return \Drupal\dwarl\ContactMethod
   *   The contact method enum.
   */
  public function validateMethod(?string $method): ContactMethod;

}
