<?php

declare(strict_types=1);

namespace Drupal\dwarl;

use Drupal\public_key_credential_source\PublicKeyCredentialSourceInterface;

/**
 * Passkey Storage Interface.
 */
interface PasskeyStorageInterface {

  /**
   * Get passkey information for current user.
   *
   * @return array
   *   Array of arrays with keys:
   *   - id
   *   - created
   *   - last_used
   *   - created_device
   *   - last_used_device
   */
  public function getPasskeys(int $userId): array;

  /**
   * Delete all passkeys for a specific user.
   *
   * @param int $userId
   *   The user id.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function deleterForUser(int $userId): void;

  /**
   * Get user passkeys.
   *
   * @param int $userId
   *   The user id.
   *
   * @return array
   *   Array of passkey data.
   */
  public function getUserPasskeys(int $userId): array;

  /**
   * Get a users single passkey.
   *
   * Throws exception if user passkey count !== 1.
   *
   * @param int $userId
   *   The user id.
   *
   * @return \Drupal\public_key_credential_source\PublicKeyCredentialSourceInterface
   *   Passkey data array.
   *
   * @throws \Drupal\dwarl\Exception\NoPasskeysException
   * @throws \Drupal\dwarl\Exception\MoreThanOnePasskeyException
   */
  public function getUserSinglePasskey(int $userId): PublicKeyCredentialSourceInterface;

  /**
   * Try to delete a passkey.
   *
   * Catch and log any fails.
   *
   * @param \Drupal\public_key_credential_source\PublicKeyCredentialSourceInterface $publicKeyCredentialSource
   *   The passkey.
   */
  public function deletePasskey(PublicKeyCredentialSourceInterface $publicKeyCredentialSource): void;

}
