<?php

declare(strict_types=1);

namespace Drupal\dwarl;

use Drupal\user\UserInterface;

/**
 * Register interface.
 */
interface RegisterInterface {

  /**
   * Create account with email.
   *
   * @param string $email
   *   The email.
   *
   * @return string
   *   The session id.
   *
   * @throws \Drupal\dwarl\Exception\DwarlStoreException
   * @throws \Drupal\dwarl\Exception\StoreExpiredException
   * @throws \Random\RandomException
   * @throws \Drupal\dwarl\Exception\DwarlPhoneNotEnabledException
   * @throws \Drupal\dwarl\Exception\FailedToGenerateOtpException
   */
  public function registerEmail(string $email): string;

  /**
   * Create account with phone.
   *
   * @param string $phone
   *   The phone.
   * @param string $country
   *   The country code.
   *
   * @return string
   *   The session id.
   *
   * @throws \Drupal\dwarl\Exception\DwarlStoreException
   * @throws \Drupal\dwarl\Exception\StoreExpiredException
   * @throws \Random\RandomException
   * @throws \Drupal\dwarl\Exception\FailedToGenerateOtpException
   * @throws \Drupal\dwarl\Exception\DwarlPhoneNotEnabledException
   */
  public function registerPhone(string $phone, string $country): string;

  /**
   * Create new user account.
   *
   * @param string $userHandle
   *   The user handle.
   * @param string|null $email
   *   The email.
   * @param string|null $phone
   *   The phone.
   * @param string|null $country
   *   The phone country.
   *
   * @return \Drupal\user\UserInterface
   *   The new user account.
   */
  public function createUser(string $userHandle, ?string $email, ?string $phone, ?string $country = NULL): UserInterface;

}
