<?php

declare(strict_types=1);

namespace Drupal\dwarl\Store;

/**
 * Store interface.
 */
interface StoreInterface {

  /**
   * Retrieves a value from this PrivateTempStore for a given key.
   *
   * @param string $key
   *   The key of the data to retrieve.
   *
   * @return mixed
   *   The data associated with the key, or NULL if the key does not exist.
   */
  public function get($key);

  /**
   * Set value.
   *
   * Log exception on fail.
   *
   * @param string $key
   *   The key.
   * @param mixed $value
   *   The value.
   *
   * @return $this
   *
   * @throws \Drupal\dwarl\Exception\DwarlStoreException
   *   If failed to set value.
   */
  public function set(string $key, $value): AnonymousStoreInterface;

}
