<?php

declare(strict_types=1);

namespace Drupal\dwarl\Util;

/**
 * Generate a random string.
 */
final class CodeGenerator {

  public const KEYSPACE_SIMPLE = 'ABCDEFGHJKMNPRSTUVWXWZ23456789';
  public const KEYSPACE_STANDARD = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMONMPQURSUVWXYZ1234567890';
  public const KEYSPACE_COMPLEX = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMONMPQURSUVWXYZ1234567890-=!@#$%^&*()_+|{}[]';

  /**
   * Generate code.
   *
   * @param int $length
   *   The length.
   * @param string $keyspace
   *   The characters available to select from.
   *
   * @return string
   *   The code.
   *
   * @todo make static
   *
   * @throws \Random\RandomException
   */
  public function generateCode(
    int $length = 6,
    string $keyspace = self::KEYSPACE_SIMPLE,
  ): string {
    if ($length < 1) {
      throw new \RangeException('Length must be a positive integer');
    }

    $keyspace = str_shuffle($keyspace);

    $pieces = [];
    $max = mb_strlen($keyspace, '8bit') - 1;
    for ($i = 0; $i < $length; ++$i) {
      $pieces[] = $keyspace[random_int(0, $max)];
    }
    return implode('', $pieces);
  }

}
