<?php

declare(strict_types=1);

namespace Drupal\dwarl\Util;

/**
 * Phone number helper methods.
 */
class PhoneNumber {

  /**
   * Format phone number suitable for database storage.
   *
   * @param string $rawPhoneNumber
   *   The raw phone number.
   * @param string $country
   *   The country.
   *
   * @return string
   *   The formatted phone number.
   *
   * @throws \Drupal\phone_number\Exception\CountryException
   *    Thrown if phone number is not valid because its country and the country
   *    provided do not match.
   * @throws \Drupal\phone_number\Exception\ParseException
   *    Thrown if phone number could not be parsed, and is thus invalid.
   * @throws \Drupal\phone_number\Exception\TypeException
   *    Thrown if phone number is an invalid type.
   */
  public static function getFormattedPhoneNumber(string $rawPhoneNumber, string $country = 'GB'): string {
    if (\Drupal::hasService('phone_number.util') === FALSE) {
      throw new \RuntimeException('Must enable phone_number module.');
    }

    /** @var \Drupal\phone_number\PhoneNumberUtil $phoneUtil */
    $phoneUtil = \Drupal::service('phone_number.util');

    $phoneUtil->testPhoneNumber($rawPhoneNumber, $country);
    $phoneNumber = $phoneUtil->getPhoneNumber($rawPhoneNumber, $country);
    $uri = $phoneNumber ? $phoneUtil->getRfc3966Uri($phoneNumber) : NULL;
    return str_replace(['tel:', '-'], '', $uri);
  }

}
