<?php

declare(strict_types=1);

namespace Drupal\dwarl\WebAuthn;

use Drupal\user\UserInterface;
use Webauthn\PublicKeyCredentialSource as PKCredentialSource;

/**
 * Authenticate request and return a JWT token.
 */
class AuthenticateRequest extends WebAuthnBase implements AuthenticateRequestInterface {

  /**
   * {@inheritdoc}
   */
  public function execute(string $userHandle, string $authenticationRequest): UserInterface {

    $pkCredentialSource = $this->webauthn->authenticateRequest($userHandle, $authenticationRequest);

    if (!$pkCredentialSource instanceof PKCredentialSource) {
      throw new \RuntimeException('Failed to get PKCredentialSource.');
    }

    $uuid = $pkCredentialSource->userHandle;
    $user = $this->userService->loadByUuid($uuid);

    if (!$user instanceof UserInterface) {
      throw new \RuntimeException('Failed to load user.');
    }

    // @todo check if user is blocked here?
    user_login_finalize($user);

    return $user;
  }

}
