<?php

declare(strict_types=1);

namespace Drupal\dwarl\WebAuthn;

use Drupal\user\UserInterface;

/**
 * Register Device Interface.
 */
interface RegisterDeviceInterface {

  /**
   * Register device.
   *
   * @param string $attestation
   *   The attestation.
   *
   * @return \Drupal\user\UserInterface
   *   The user.
   *
   * @throws \Drupal\webauthn_jwt\Exception\WebauthnJwtException
   * @throws \Drupal\public_key_credential_source\Exception\PublicKeyCredentialSourceWebauthnInvalidArgumentException
   * @throws \Drupal\dwarl\Exception\DwarlStoreException
   * @throws \Drupal\dwarl\Exception\UserSaveException
   * @throws \Drupal\Core\TempStore\TempStoreException
   */
  public function register(string $attestation): UserInterface;

  /**
   * Add passkey.
   *
   * @param string $attestation
   *   Attestation.
   *
   * @return \Drupal\user\UserInterface
   *   The user.
   *
   * @throws \Drupal\public_key_credential_source\Exception\PublicKeyCredentialSourceWebauthnInvalidArgumentException
   */
  public function addPasskey(string $attestation): UserInterface;

}
