<?php

declare(strict_types=1);

namespace Drupal\dwarl\WebAuthn;

use Drupal\Core\Session\AccountProxyInterface;
use Drupal\dwarl\Store\AnonymousStoreInterface;
use Drupal\dwarl\UserService;
use Drupal\public_key_credential_source\Services\PublicKeyCredentialSourceWebauthn;
use Drupal\user\Entity\User;
use Drupal\webauthn_framework\Services\Webauthn;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Register Device Options.
 */
class RegisterDeviceOptions extends WebAuthnBase implements RegisterDeviceOptionsInterface {

  /**
   * Constructs an WebauthnJwtBase.
   */
  public function __construct(
    AccountProxyInterface $currentUser,
    PublicKeyCredentialSourceWebauthn $webauthn,
    LoggerInterface $logger,
    AnonymousStoreInterface $store,
    UserService $userService,
    RequestStack $requestStack,
    private readonly Webauthn $webauthnService,
  ) {
    parent::__construct($currentUser, $webauthn, $logger, $store, $userService, $requestStack);
  }

  /**
   * {@inheritdoc}
   */
  public function getRegistrationOptions(): string {
    $user = User::load(\Drupal::currentUser()->id());
    if (!$user || $user->isAnonymous()) {
      $user = $this->store->getUser();
    }

    $context = ['@uid' => $user->id()];
    $this->logger->debug('Generating device token options for user @uid', $context);

    $options = $this->webauthn->getRegistrationOptions($user);

    return $this->webauthnService->serializePublicKeyCredential($options);
  }

}
