<?php

declare(strict_types=1);

namespace Drupal\dwarl_test\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\dwarl\Authentication\Provider\DwarlJwtAuth;
use Drupal\jwt\Authentication\Provider\JwtAuth;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Returns responses for Decoupled Webauthn Register Login test module routes.
 */
final class DwarlTestController extends ControllerBase {

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request|null
   */
  private ?Request $request;

  /**
   * The controller constructor.
   */
  public function __construct(
    private readonly JwtAuth $auth,
    RequestStack $requestStack,
  ) {
    $this->request = $requestStack->getCurrentRequest();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('jwt.authentication.jwt'),
      $container->get('request_stack'),
    );
  }

  /**
   * Builds the response.
   *
   * This method is such a security risk and I don't like it.
   * But my functional test needs it.
   */
  public function registerDeviceTest(): JsonResponse {

    $email = $this->request->get('user_handle');

    $user = user_load_by_mail($email);

    $user->activate()->save();

    user_login_finalize($user);
    $token = $this->auth->generateToken();
    $response = ['token' => $token];

    return new JsonResponse($response);
  }

}
