<?php

declare(strict_types=1);

namespace Drupal\dwarl_test;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\DependencyInjection\ServiceProviderBase;
use Drupal\dwarl_test\Detector\TestDetector;
use Drupal\dwarl_test\WebAuthn\AuthenticateRequest;
use Drupal\dwarl_test\WebAuthn\RegisterDevice;
use Drupal\dwarl_test\WebAuthn\RegisterDeviceOptions;
use Drupal\dwarl_test\WebAuthn\RequestOptions;

class DwarlTestServiceProvider extends ServiceProviderBase {
  /**
   * {@inheritdoc}
   */
  public function alter(ContainerBuilder $container): void {
    if ($container->hasDefinition('dwarl.register_device')) {
      $definition = $container->getDefinition('dwarl.register_device');
      $definition->setClass(RegisterDevice::class);
    }
    if ($container->hasDefinition('dwarl.register_device_options')) {
      $definition = $container->getDefinition('dwarl.register_device_options');
      $definition->setClass(RegisterDeviceOptions::class);
    }
    if ($container->hasDefinition('dwarl.authenticate_request')) {
      $definition = $container->getDefinition('dwarl.authenticate_request');
      $definition
        ->setClass(AuthenticateRequest::class);
    }
    if ($container->hasDefinition('dwarl.request_options')) {
      $definition = $container->getDefinition('dwarl.request_options');
      $definition
        ->setClass(RequestOptions::class);
    }
    if ($container->hasDefinition('universal_device_detection.default')) {
      $definition = $container->getDefinition('universal_device_detection.default');
      $definition
        ->setClass(TestDetector::class);
    }
  }

}
