<?php

declare(strict_types=1);

namespace Drupal\dwarl_test\EventSubscriber;

use Drupal\Core\TempStore\PrivateTempStore;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\dwarl\Event\SendOtpEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Dwarl test send OTP event subscriber.
 */
class SendOtpEventSubscriber implements EventSubscriberInterface {

  /**
   * Dwarl private temp store.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStore
   */
  private PrivateTempStore $tempStore;

  /**
   * SendOtpEventSubscriber constructor.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $privateTempStoreFactory
   *   Private temp store factory.
   */
  public function __construct(
    PrivateTempStoreFactory $privateTempStoreFactory,
  ) {
    $this->tempStore = $privateTempStoreFactory->get('dwarl_test');
  }

  /**
   * {@inheritDoc}
   */
  public static function getSubscribedEvents(): array {
    $events = [];
    $events[SendOtpEvent::SEND_OTP][] = ['sendOtp'];
    return $events;
  }

  /**
   * Send otp event.
   *
   * Store the plain otp in the db so the test script can get it.
   *
   * @param \Drupal\dwarl\Event\SendOtpEvent $sendOtpEvent
   *   The send otp event.
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   */
  public function sendOtp(SendOtpEvent $sendOtpEvent): void {
    $otp = $sendOtpEvent->getOtp();

    \Drupal::request()->getSession()->set('core.tempstore.private.owner', 'dwarl_test');
    $this->tempStore->set('otp', $otp);
    $this->tempStore->set('email', $sendOtpEvent->getEmail());
    $this->tempStore->set('phone', $sendOtpEvent->getPhone());
    $this->tempStore->set('method', $sendOtpEvent->getMethod());
  }

}
