<?php

declare(strict_types=1);

namespace Drupal\Tests\dwarl\Functional;

use Drupal\dwarl\ContactMethod;
use Drupal\Tests\dwarl\Traits\ForgotPasswordTrait;

/**
 * Forgot password tests.
 *
 * See https://cheatsheetseries.owasp.org/cheatsheets/Forgot_Password_Cheat_Sheet.html
 */
class ForgotPasswordTest extends TestBase {
  use ForgotPasswordTrait;

  /**
   * Forgot password with email without a passkey.
   */
  public function testForgotPasswordWithEmailWithoutPasskey() {
    $email = 'email@dwarl-example.com';
    $password = 'ClearAirTurbulence';
    $this->dwarlCreateUser($email, ['validated_email'], $password);

    $sessionId = $this->forgotPassword($email);
    $otp = $this->getOtp($email, ContactMethod::Email);
    $data = $this->forgotPasswordSetPassword($password, $otp, $sessionId);
    $this->assertArrayHasKey('token', $data);
  }

  /**
   * Forgot password with email with a passkey.
   */
  public function testForgotPasswordWithEmailWithPasskey() {
    $email = 'email@dwarl-example.com';
    $password = 'ClearAirTurbulence';
    $this->dwarlCreateUser($email, ['validated_email'], $password, 1);

    $sessionId = $this->forgotPassword($email);
    $otp = $this->getOtp($email, ContactMethod::Email);
    $data = $this->forgotPasswordSetPassword($password, $otp, $sessionId);
    $this->assertArrayHasKey('requestOptions', $data);
    $authRequest = $data['requestOptions']['challenge'];
    $this->forgotPasswordSetPasswordConfirm($password, $authRequest, $sessionId);
  }

  /**
   * Forgot password with phone without a passkey.
   */
  public function testForgotPasswordWithPhoneWithoutPasskey() {
    $phone = $this->randomPhone();
    $password = 'ClearAirTurbulence';
    $user = $this->dwarlCreateUser($phone, ['validated_phone'], $password);

    $hasField = $user->hasField('field_preferred_contact_method');
    $this->assertTrue($hasField, 'Has field_preferred_contact_method');

    $sessionId = $this->forgotPassword($phone);
    $otp = $this->getOtp($phone, ContactMethod::Phone);
    $data = $this->forgotPasswordSetPassword($password, $otp, $sessionId);
    $this->assertArrayHasKey('token', $data);
  }

  /**
   * Forgot password with phone with a passkey.
   */
  public function testForgotPasswordWithPhoneWithPasskey() {
    $phone = $this->randomPhone();
    $password = 'ClearAirTurbulence';
    $this->dwarlCreateUser($phone, ['validated_phone'], $password, 1);

    $sessionId = $this->forgotPassword($phone);
    $otp = $this->getOtp($phone, ContactMethod::Phone);
    $data = $this->forgotPasswordSetPassword($password, $otp, $sessionId);
    $this->assertArrayHasKey('requestOptions', $data);
    $authRequest = $data['requestOptions']['challenge'];
    $this->forgotPasswordSetPasswordConfirm($password, $authRequest, $sessionId);
  }

}
