<?php

declare(strict_types=1);

namespace Drupal\Tests\dwarl\Functional;

use Drupal\Tests\dwarl\Traits\LoginTrait;
use Drupal\Tests\dwarl\Traits\WebAuthnTrait;

/**
 * Login tests.
 */
class LoginTest extends TestBase {
  use LoginTrait;
  use WebAuthnTrait;

  /**
   * Login with email and password.
   */
  public function testLoginWithEmailPassword() {
    $email = 'email@dwarl-example.com';
    $password = 'ClearAirTurbulence';
    $this->dwarlCreateUser($email, ['validated_email'], $password);

    $data = $this->authenticatePassword($email, $password);
    $this->assertArrayHasKey('token', $data['body'], 'authenticate_password has token');
    $jwtToken = $data['body']['token'];

    $this->getUserInfo($jwtToken);
  }

  /**
   * Login with email and password with passkey.
   */
  public function testLoginWithEmailPasswordWithPasskey() {
    $email = 'email@dwarl-example.com';
    $password = 'ClearAirTurbulence';
    $this->dwarlCreateUser($email, ['validated_email'], $password, 1);

    $data = $this->authenticatePassword($email, $password, TRUE);
    $this->assertArrayHasKey('requestOptions', $data['body'], 'authenticate_password has requestOptions');
    $this->assertArrayHasKey('Session-Id', $data['headers'], 'authenticate_password has session id');

    $authRequest = $data['body']['requestOptions']['challenge'];
    $sessionId = $data['headers']['Session-Id'][0];

    $jwtToken = $this->authenticateRequest($email, $authRequest, $sessionId);
    $this->assertNotEmpty($jwtToken, 'There is a JWT token');

    $this->getUserInfo($jwtToken);
  }

  /**
   * Login with email and passkey.
   */
  public function testLoginWithEmailPasskey() {
    $email = 'email@dwarl-example.com';
    $this->dwarlCreateUser($email, ['validated_email'], NULL, 1);

    $response = $this->requestOptions($email);
    $jwtToken = $this->authenticateRequest($email, $response['authentication_request'], $response['session_id']);
    $this->assertNotEmpty($jwtToken, 'There is a JWT token');

    $this->getUserInfo($jwtToken);
  }

  /**
   * Login with phone and password.
   */
  public function testLoginWithPhonePassword() {
    $phone = $this->randomPhone();
    $password = 'ClearAirTurbulence';
    $this->dwarlCreateUser($phone, ['validated_phone'], $password);

    $data = $this->authenticatePassword($phone, $password);
    $this->assertArrayHasKey('token', $data['body'], 'authenticate_password has token');
    $jwtToken = $data['body']['token'];

    $this->getUserInfo($jwtToken);
  }

  /**
   * Login with phone and password with passkey.
   */
  public function testLoginWithPhonePasswordWithPasskey() {
    $phone = $this->randomPhone();
    $password = 'ClearAirTurbulence';
    $this->dwarlCreateUser($phone, ['validated_phone'], $password, 1);

    $data = $this->authenticatePassword($phone, $password, TRUE);
    $this->assertArrayHasKey('requestOptions', $data['body'], 'authenticate_password has requestOptions');
    $this->assertArrayHasKey('Session-Id', $data['headers'], 'authenticate_password has session id');

    $authRequest = $data['body']['requestOptions']['challenge'];
    $sessionId = $data['headers']['Session-Id'][0];

    $jwtToken = $this->authenticateRequest($phone, $authRequest, $sessionId);
    $this->assertNotEmpty($jwtToken, 'There is a JWT token');

    $this->getUserInfo($jwtToken);
  }

  /**
   * Login with phone and passkey.
   */
  public function testLoginWithPhonePasskey() {
    $phone = $this->randomPhone();
    $this->dwarlCreateUser($phone, ['validated_email'], NULL, 1);

    $response = $this->requestOptions($phone);
    $jwtToken = $this->authenticateRequest($phone, $response['authentication_request'], $response['session_id']);
    $this->assertNotEmpty($jwtToken, 'There is a JWT token');

    $this->getUserInfo($jwtToken);
  }

}
