<?php

declare(strict_types=1);

namespace Drupal\Tests\dwarl\Functional;

use Drupal\Tests\dwarl\Traits\ManagePasskeysTrait;

/**
 * Manage passkey tests.
 */
class ManagePasskeysTest extends TestBase {
  use ManagePasskeysTrait;

  /**
   * Add passkey to an account using an email.
   */
  public function testAddPasskeyWithEmail(): void {
    $email = 'test@dwarl-example.com';
    $password = 'changeMe';
    $user = $this->dwarlCreateUser($email, ['validated_email'], $password);
    $token = $this->getJwtToken($user);

    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
    $this->assertEmpty($passkeys, 'No passkeys at start of test');

    $this->addPasskey($token, TRUE);

    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
    $this->assertSame(1, count($passkeys), 'user has one 1 passkey');

    $this->addPasskey($token, FALSE);

    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
    $this->assertSame(2, count($passkeys), 'User has 2 passkeys');
  }

  /**
   * Add passkey to an account using an phone.
   */
  public function testAddPasskeyWithPhone() {
    $phone = $this->randomPhone();
    $password = 'changeMe';
    $user = $this->dwarlCreateUser($phone, ['validated_phone'], $password);
    $token = $this->getJwtToken($user);

    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
    $this->assertEmpty($passkeys, 'No passkeys at start of test');

    $this->addPasskey($token, TRUE);

    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
    $this->assertSame(1, count($passkeys), '1 passkey at end of test');
  }

  /**
   * Delete a passkey.
   */
  public function testDeletingPasskey() {
    $email = 'test@dwarl-example.com';
    $password = 'changeMe';
    $user = $this->dwarlCreateUser($email, ['validated_email'], $password, 1);
    $token = $this->getJwtToken($user);

    $info = $this->getUserInfo($token);
    $passkeyId = (int) $info['passkeys'][0]['id'];

    $this->deletePasskey($token, $passkeyId);

    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
    $this->assertSame(0, count($passkeys), '0 passkeys at end of test');
  }

}
