<?php

namespace Drupal\Tests\dwarl;

class StartRegistration {

  public static function startRegistration(array $optionsJson) {
    //    const { optionsJSON, useAutoRegister = false } = options;
    $optionsJson['useAutoRegister'] = FALSE;

    //  // We need to convert some values to Uint8Arrays before passing the credentials to the navigator
    //  const publicKey: PublicKeyCredentialCreationOptions = {
    //      ...optionsJSON,
    //    challenge: base64URLStringToBuffer(optionsJSON.challenge),
    //    user: {
    //        ...optionsJSON.user,
    //      id: base64URLStringToBuffer(optionsJSON.user.id),
    //    },
    //    excludeCredentials: optionsJSON.excludeCredentials?.map(
    //        toPublicKeyCredentialDescriptor,
    //      ),
    //  };

    $publicKey = [
      ...$optionsJson,
      'challenge' => self::base64URLStringToBuffer($optionsJson['challenge']),
      'user' => [
        ...$optionsJson['user'],
        'id' => self::base64URLStringToBuffer($optionsJson['user']['id']),
      ],
      'excludeCredentials' => [
        !empty($optionsJson['excludeCredentials'])
          ? array_map(
            [self::class, 'toPublicKeyCredentialDescriptor'],
            $optionsJson['excludeCredentials']
        )
          : NULL
      ],
    ];

    //  // Prepare options for `.create()`
    //  const createOptions: CredentialCreationOptions = {};
    $createOptions = [];
    //
    //  /**
    //   * Try to use conditional create to register a passkey for the user with the password manager
    //   * the user just used to authenticate with. The user won't be shown any prominent UI by the
    //   * browser.
    //   */
    //  if (useAutoRegister) {
    //    // @ts-ignore: `mediation` doesn't yet exist on CredentialCreationOptions but it's possible as of Sept 2024
    //    createOptions.mediation = 'conditional' as CredentialMediationRequirement;
    //  }


    //
    //  // Finalize options
    //  createOptions.publicKey = publicKey;

    $createOptions['publicKey'] = $publicKey;

    //  // Set up the ability to cancel this request if the user attempts another
    //  createOptions.signal = WebAuthnAbortService.createNewAbortSignal();
    //
    //  // Wait for the user to complete attestation
    //  let credential;
    //  try {
    //    credential = (await navigator.credentials.create(createOptions)) as RegistrationCredential;
    //  } catch (err) {
    //    throw identifyRegistrationError({ error: err as Error, options: createOptions });
    //  }
    //
    //  if (!credential) {
    //    throw new Error('Registration was not completed');
    //  }
    //
    //  const { id, rawId, response, type } = credential;
    //
    //  // Continue to play it safe with `getTransports()` for now, even when L3 types say it's required
    //  let transports: AuthenticatorTransportFuture[] | undefined = undefined;
    //  if (typeof response.getTransports === 'function') {
    //      transports = response.getTransports();
    //    }
    //
    //  // L3 says this is required, but browser and webview support are still not guaranteed.
    //  let responsePublicKeyAlgorithm: number | undefined = undefined;
    //  if (typeof response.getPublicKeyAlgorithm === 'function') {
    //      try {
    //        responsePublicKeyAlgorithm = response.getPublicKeyAlgorithm();
    //      } catch (error) {
    //        warnOnBrokenImplementation('getPublicKeyAlgorithm()', error as Error);
    //    }
    //    }
    //
    //  let responsePublicKey: string | undefined = undefined;
    //  if (typeof response.getPublicKey === 'function') {
    //      try {
    //        const _publicKey = response.getPublicKey();
    //        if (_publicKey !== null) {
    //          responsePublicKey = bufferToBase64URLString(_publicKey);
    //        }
    //      } catch (error) {
    //        warnOnBrokenImplementation('getPublicKey()', error as Error);
    //    }
    //    }
    //
    //  // L3 says this is required, but browser and webview support are still not guaranteed.
    //  let responseAuthenticatorData: string | undefined;
    //  if (typeof response.getAuthenticatorData === 'function') {
    //      try {
    //        responseAuthenticatorData = bufferToBase64URLString(
    //          response.getAuthenticatorData(),
    //        );
    //      } catch (error) {
    //        warnOnBrokenImplementation('getAuthenticatorData()', error as Error);
    //    }
    //    }
    //
    //  return {
    //      id,
    //    rawId: bufferToBase64URLString(rawId),
    //    response: {
    //        attestationObject: bufferToBase64URLString(response.attestationObject),
    //      clientDataJSON: bufferToBase64URLString(response.clientDataJSON),
    //      transports,
    //      publicKeyAlgorithm: responsePublicKeyAlgorithm,
    //      publicKey: responsePublicKey,
    //      authenticatorData: responseAuthenticatorData,
    //    },
    //    type,
    //    clientExtensionResults: credential.getClientExtensionResults(),
    //    authenticatorAttachment: toAuthenticatorAttachment(
    //        credential.authenticatorAttachment,
    //      ),
    //  };

  }

  /**
   * Convert from a Base64URL-encoded string to an Array Buffer. Best used when
   * converting a credential ID from a JSON string to an ArrayBuffer, like in
   * allowCredentials or excludeCredentials
   *
   * Helper method to compliment `bufferToBase64URLString`
   */
  protected static function base64URLStringToBuffer(string $base64URLString) {
    // Convert from Base64URL to Base64
    //  const base64 = base64URLString.replace(/-/g, '+').replace(/_/g, '/');
    $base64 = str_replace(['-', '_'], ['+', '/'], $base64URLString);
    /**
     * Pad with '=' until it's a multiple of four
     * (4 - (85 % 4 = 1) = 3) % 4 = 3 padding
     * (4 - (86 % 4 = 2) = 2) % 4 = 2 padding
     * (4 - (87 % 4 = 3) = 1) % 4 = 1 padding
     * (4 - (88 % 4 = 0) = 4) % 4 = 0 padding
     */
    //  const padLength = (4 - (base64.length % 4)) % 4;
    $padLength = (4 - (strlen($base64) % 4)) % 4;
    //  const padded = base64.padEnd(base64.length + padLength, '=');
    $padded = str_pad($base64, strlen($base64) + $padLength, '=');

    // Convert to a binary string
    //  const binary = atob(padded);
    $binary = base64_decode($padded);

    // Convert binary string to buffer
    //  const buffer = new ArrayBuffer(binary.length);
    $buffer = [];
    //  const bytes = new Uint8Array(buffer);
    //
    //  for (let i = 0; i < binary.length; i++) {
    //    bytes[i] = binary.charCodeAt(i);
    //  }

    $binary = '';
    for ($index = 0; $index < strlen($binary); $index++) {
      $binary = $binary . pack("C*", substr($binary, $index, 1));
    }

    return $binary;
    //return buffer;
  }

  protected static function toPublicKeyCredentialDescriptor(
    array $descriptor,
) {
//  const { id } = descriptor;
    $id = $descriptor['id'];

//return {
//  ...descriptor,
//    id: base64URLStringToBuffer(id),
//    /**
//     * `descriptor.transports` is an array of our `AuthenticatorTransportFuture` that includes newer
//     * transports that TypeScript's DOM lib is ignorant of. Convince TS that our list of transports
//     * are fine to pass to WebAuthn since browsers will recognize the new value.
//     */
//    transports: descriptor.transports as AuthenticatorTransport[],
//  };

    return [
      ...$descriptor,
      'id' => self::base64URLStringToBuffer($id)
    ];
}

}
