<?php

declare(strict_types=1);

namespace Drupal\Tests\dwarl\Traits;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Url;
use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;
use GuzzleHttp\RequestOptions;

/**
 * Methods for login tests.
 */
trait LoginTrait {

  /**
   * Authenticate password.
   *
   * @param string $userHandle
   *   User handle.
   * @param string $password
   *   Password.
   * @param bool $addPermission
   *   Should permissions be set and tested?
   *
   * @return array
   *   Body and headers.
   */
  protected function authenticatePassword(
    string $userHandle,
    string $password,
    bool $addPermission = TRUE
  ): array {
    $url = Url::fromRoute('dwarl.authenticate_password');
    $request_options = [
      RequestOptions::FORM_PARAMS => [
        'user_handle' => $userHandle,
        'password' => $password,
      ],
    ];
    $response = $this->makeApiRequest('POST', $url, $request_options);
    $statusCode = $response->getStatusCode();

    if ($addPermission) {
      $this->assertSame(403, $statusCode, 'authenticate_password response code without permission');

      // Assign 'dwarl login with password' permission.
      $anon_role = Role::load(RoleInterface::ANONYMOUS_ID);
      $this->grantPermissions($anon_role, ['dwarl login with password']);
    }

    $response = $this->makeApiRequest('POST', $url, $request_options);
    $statusCode = $response->getStatusCode();
    $this->assertSame(200, $statusCode, 'authenticate_password response code');

    $data = Json::decode($response->getBody());
    $headers = $response->getHeaders();

    return [
      'body' => $data,
      'headers' => $headers,
    ];
  }

}
