<?php
//
//declare(strict_types=1);
//
//namespace Drupal\Tests\dwarl\Functional;
//
//use Drupal\Core\Url;
//use Drupal\dwarl\ContactMethod;
//use GuzzleHttp\RequestOptions;
//
//class DwarlEmailTest extends TestBase {
//
//  public function testDwarlEmail() {
//    $email = $this->randomMachineName() . '@dwarl.com';
//
//    // Register user 1 with passkey.
//    $jwtToken = $this->registerWithPasskey($email);
//
//    // Check can fetch user info.
//    $this->fetchUserInfo($jwtToken, $email);
//
//    // Login with passkey.
//    $jwtToken = $this->loginWithPasskey($email);
//    // Validate token works.
//    $this->getUserInfo($jwtToken);
//
//    // Lost passkey flow (without password).
//    $jwtToken = $this->lostPasskeyWithoutPassword($email);
//
//    // Set password with passkey.
//    $password = 'SuperSecurePassword';
//    $this->setPasswordWithPasskey($jwtToken, $password);
//    $this->checkUserHasPassword($email);
//
//    // Lost passkey flow (with password).
//    $jwtToken = $this->lostPasskeyWithPassword($email, $password);
//
//    // Add second passkey.
//    $this->addPasskey($jwtToken, $email);
//
//    // User should have 2 passkey.
//    $user = $this->dwarlUser()->loadByUserHandle($email);
//    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
//    $this->assertEquals(2, count($passkeys), 'There are 2 passkeys');
//    $this->assertPasskeyIsValid($passkeys[1]);
//
//    // Lost passkey flow (with multiple passkeys).
//    $this->lostPasskeyFlowWithMultiplePasskeys($email);
//
//
//
//    // Register user 2 with password.
//    $email = $this->randomMachineName() . '@dwarl.com';
//    $password = 'GNUTerryPratchett';
//    $jwtToken = $this->resisterWithPassword($email, $password);
//    $this->getUserInfo($jwtToken);
//
//    // Login with password.
//    $jwtToken = $this->loginWithPasswordWithoutPasskey($email, $password);
//    $this->getUserInfo($jwtToken);
//
//    // Forgot password flow (without passkey).
//    $password = 'electricBoogaloo2';
//    $jwtToken = $this->forgotPasswordWithoutPasskey($email, $password);
//
//    // Create passkey logged in.
//    $this->addPasskey($jwtToken, $email, FALSE);
//
//    // Login with password (with passkey)
//    $jwtToken = $this->loginWithPasswordWithPasskey($email, $password);
//    $this->getUserInfo($jwtToken);
//
//    // Forgot password flow (with passkey).
//    $password = 'ClearAirTurbulence';
//    $jwtToken = $this->forgotPasswordWithPasskey($email, $password);
//    $this->getUserInfo($jwtToken);
//
//    // Register user 3 with passkey.
//    $email3 = $this->randomMachineName() . '@dwarl.com';
//    $token = $this->registerWithPasskey($email3, FALSE);
//
//    // Set password with otp.
//    $password = 'wizzPopBang';
//    $this->setPasswordWithOtpWithoutPassword($password, $token, $email3);
//
////    $this->setPasswordWithOtpWithPassword($password, $token);
//
//    return;
//
//    // Delete passkey.
//    $this->deletePasskey();
//
//    // Test otp generation rate limit.
//
//  }
//
//  private function registerWithPasskey(string $email, bool $setPermission = TRUE)
//  {
//    $user = user_load_by_mail($email);
//    $this->assertFalse($user, "user $email does not exist at start of test");
//
//    // Create user.
//    $sessionId = $this->createUserEmail($email, $setPermission);
//
//    // Get users otp code.
//    $otp = $this->getOtp($email, ContactMethod::Email);
//
//    // Validate email.
//    $this->validateContactMethod($email, $sessionId, $otp);
//
//    // Create passkey.
//    $optionsJson = $this->registerDeviceOptions($sessionId, 'dwarl.registration.register_device_options');
//    $jwtToken = $this->registerDevice($sessionId, $email, $optionsJson, 'dwarl.registration.register_device');
//
//    $this->assertNotEmpty($jwtToken, 'There is a JWT token');
//
//    // User should have 1 passkey.
//    $user = $this->dwarlUser()->loadByUserHandle($email);
//    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
//    $this->assertPasskeyIsValid($passkeys[0]);
//
//    return $jwtToken;
//  }
//
//  private function fetchUserInfo(string $jwtToken, string $email)
//  {
//    $userInfo = $this->getUserInfo($jwtToken);
//    $this->validateUserInfo($userInfo, $email);
//  }
//
//  private function loginWithPasskey(string $email)
//  {
//    $response = $this->requestOptions($email);
//    $jwtToken = $this->authenticateRequest($email, $response['authentication_request'], $response['session_id']);
//    $this->assertNotEmpty($jwtToken, 'There is a JWT token');
//    return $jwtToken;
//  }
//
//  private function lostPasskeyWithoutPassword(string $email)
//  {
//    // Lost passkey flow.
//    $sessionId = $this->lostPasskey($email);
//    $otp = $this->getOtp($email, ContactMethod::Email);
//    $status = $this->lostPasskeySubmitOtpOld($otp, $sessionId);
//
//    $this->assertSame('set auth method', $status, 'status should be set auth method as no password is set');
//
//    // Users passkey should have been deleted at this point.
//    $user = $this->dwarlUser()->loadByUserHandle($email);
//    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
//    $this->assertEmpty($passkeys, 'passkeys should be empty');
//
//    // We could now set a passkey or a password.
//    // Let's create a passkey.
//    $optionsJson = $this->registerDeviceOptions($sessionId, 'dwarl.lost_passkey.register_device_options');
//
//    $jwtToken = $this->registerDevice($sessionId, $email, $optionsJson, 'dwarl.lost_passkey.register_device');
//
//    // User should now 1 have passkey again.
//    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
//    $this->assertPasskeyIsValid($passkeys[0]);
//
//    return $jwtToken;
//  }
//
//  private function setPasskeyWithPassword()
//  {
//
//  }
//
//
//  private function checkUserHasPassword(string $email)
//  {
//    drupal_flush_all_caches();
//    $user = $this->dwarlUser()->loadByUserHandle($email);
//    $this->assertSame(60, strlen($user->getPassword()), 'user password should be 60 characters');
//  }
//
//  private function lostPasskeyWithPassword(string $email, string $password)
//  {
//    $sessionId = $this->lostPasskey($email);
//    $otp = $this->getOtp($email, ContactMethod::Email);
//    $status = $this->lostPasskeySubmitOtpOld($otp, $sessionId);
//    $this->assertSame('supply password', $status, 'supply password');
//
//    // Users passkey should NOT have been deleted at this point.
//    $user = $this->dwarlUser()->loadByUserHandle($email);
//    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
//    $this->assertNotEmpty($passkeys, 'passkeys should NOT be empty');
//
//    $this->lostPasskeySubmitPassword($password, $sessionId);
//
//    // Users passkey should have been deleted at this point.
//    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
//    $this->assertEmpty($passkeys, 'passkeys should be empty');
//
//    // We could now set a passkey or a password.
//    // Let's create a passkey.
//    $optionsJson = $this->registerDeviceOptions($sessionId, 'dwarl.lost_passkey.register_device_options');
//
//    $jwtToken = $this->registerDevice($sessionId, $email, $optionsJson, 'dwarl.lost_passkey.register_device');
//
//    // User should now 1 have passkey again.
//    $passkeys = $this->passkeyStorage()->getPasskeys((int) $user->id());
//    $this->assertPasskeyIsValid($passkeys[0]);
//
//    return $jwtToken;
//  }
//
//  private function addPasskey(string $token, string $email, bool $addPermission = TRUE)
//  {
//    $data = $this->registerDeviceOptionsAuthenticated($token, $addPermission);
//    $this->registerDeviceAuthenticated($token, $data);
//  }
//
//  private function lostPasskeyFlowWithMultiplePasskeys(string $email)
//  {
//    $sessionId = $this->lostPasskey($email);
//    $otp = $this->getOtp($email, ContactMethod::Email);
//    $error = $this->lostPasskeySubmitOtpOld($otp, $sessionId, FALSE);
//    $this->assertSame(
//      'Account has multiple passkeys. Use a different passkey to sign in.',
//      $error, 'multiple passkeys error');
//  }
//
//  private function resisterWithPassword(string $email, string $password)
//  {
//    $user = user_load_by_mail($email);
//    $this->assertFalse($user, "user $email does not exist at start of test");
//
//    // Create user.
//    $sessionId = $this->createUserEmail($email, FALSE);
//
//    // Get users otp code.
//    $otp = $this->getOtp($email, ContactMethod::Email);
//
//    // Validate email.
//    $this->validateContactMethod($email, $sessionId, $otp);
//
//    // Create password.
//    $jwtToken = $this->registerSetPassword($password, $sessionId);
//
//    return $jwtToken;
//  }
//
//  private function loginWithPasswordWithoutPasskey(string $email, string $password)
//  {
//    $data = $this->authenticatePassword($email, $password);
//    $this->assertArrayHasKey('token', $data['body'], 'authenticate_password has token');
//    return $data['body']['token'];
//  }
//
//  private function forgotPasswordWithoutPasskey(string $email, string $password)
//  {
//    $sessionId = $this->forgotPassword($email);
//    $otp = $this->getOtp($email, ContactMethod::Email);
//    $data = $this->forgotPasswordSetPassword($password, $otp, $sessionId);
//    $this->assertArrayHasKey('token', $data);
//    return $data['token'];
//  }
//
//  private function loginWithPasswordWithPasskey(string $email, string $password)
//  {
//    $data = $this->authenticatePassword($email, $password, FALSE);
//    $this->assertArrayHasKey('requestOptions', $data['body'], 'authenticate_password has requestOptions');
//    $this->assertArrayHasKey('Session-Id', $data['headers'], 'authenticate_password has session id');
//
//    $authRequest = $data['body']['requestOptions']['challenge'];
//    $sessionId = $data['headers']['Session-Id'][0];
//
//    $jwtToken = $this->authenticateRequest($email, $authRequest, $sessionId);
//    $this->assertNotEmpty($jwtToken, 'There is a JWT token');
//
//    return $jwtToken;
//  }
//
//  private function forgotPasswordWithPasskey(string $email, string $password)
//  {
//    $sessionId = $this->forgotPassword($email, FALSE);
//    $otp = $this->getOtp($email, ContactMethod::Email);
//    $data = $this->forgotPasswordSetPassword($password, $otp, $sessionId);
//
//    $this->assertArrayHasKey('requestOptions', $data);
//
//    $authRequest = $data['requestOptions']['challenge'];
//
//    return $this->forgotPasswordSetPasswordConfirm($password, $authRequest, $sessionId);
//  }
//
//  private function setPasswordWithOtpWithoutPassword(string $password, string $token, string $email)
//  {
//    $this->requestSetPasswordOtp('email', $token);
//    $otp = $this->getOtp($email, ContactMethod::Email);
//
//    print_r(['otp' => $otp]);
//
//    $this->setPasswordWithOtpRequest($password, $otp, $token);
//  }
//
//  private function setPasswordWithOtpWithPassword(string $passowrd, string $token) {
//    $sessionId = $this->requestChangePasswordOtp('email', $passowrd, $token);
//  }
//
//  private function deletePasskey()
//  {
//  }
//
//  protected function validateUserInfo(array $userInfo, string $email) {
//    $this->assertSame($email, $userInfo['email'], 'User info email matches');
//    $this->assertNull($userInfo['phone'], 'Phone is not set');
//    $this->assertNull($userInfo['preferredContactMethod'], 'preferredContactMethod is not set');
//    $this->assertFalse($userInfo['hasPassword'], 'hasPassword is false');
//    $this->assertEquals(1, count($userInfo['passkeys']), 'there is one passkey');
//    $passkey = reset($userInfo['passkeys']);
//    $this->assertPasskeyIsValid($passkey);
//  }
//
//}
