<?php

namespace Drupal\Tests\dwarl\Functional;

use Drupal\dwarl\Otp\OtpSendMethod;

class RegisterEmailPassword extends TestBase {

  /**
   * Test: Register with email and password.
   */
  public function testRegisterAndLogin() {

    // Set test email.
    $email = $this->randomMachineName() . '@dwarl.com';

    $user = user_load_by_mail($email);
    $this->assertFalse($user, "user $email does not exist at start of test");

    // Create user.
    $sessionId = $this->createUserEmail($email);

    // Get users otp code.
    $otp = $this->getOtp($email, OtpSendMethod::Email);

    // Validate email.
    $this->validateContactMethod($email, $sessionId, $otp);

    // Create password.
    $password = 'superSecretPassword';
    $jwtToken = $this->setPassword($password, $sessionId);

    // Fetch user info.
    $userInfo = $this->getUserInfo($jwtToken);
    $this->validateUserInfo($userInfo, $email);

  }

  protected function validateUserInfo(array $userInfo, string $email) {
    $this->assertSame($email, $userInfo['email'], 'User info email matches');
    $this->assertNull($userInfo['phone'], 'Phone is not set');
    $this->assertNull($userInfo['preferredContactMethod'], 'preferredContactMethod is not set');
    $this->assertTrue($userInfo['hasPassword'], 'hasPassword is true');
    $this->assertEquals(0, count($userInfo['passkeys']), 'there are no passkeys');
  }

}
